define bi_git::pull (
    $remote = $title,
    $local_dir = undef,
    $user = undef,
    $refspec = 'master',
    $verbose = undef,
    $quiet = undef,
    $commit = undef,
    $fast_forward = undef,
    $fast_forward_only = undef,
    $all = undef,
    $force = undef,
    $http_proxy = undef,
    $https_proxy = undef,
) {
    if $remote == undef or !is_string($remote) { 
        fail("'remote' must be defined") 
    }
    
    if $local_dir == undef or !is_string($local_dir) { 
        fail("'local_dir' must be defined") 
    }
    
    if $user == undef or !is_string($user) { 
        fail("'user' must be defined") 
    }
    
    if $refspec == undef or !is_string($refspec) { 
        fail("'refspec' must be defined") 
    }
    
    if $commit != undef {
        if is_bool($commit) {
            $arg_commit = "--commit"
        } else {
            fail("'commit' must be defined and a boolean")
        }  
    } else {
        $arg_commit = undef
    }
    
    if $fast_forward != undef {
        if is_bool($fast_forward) {
            $arg_fast_forward = "--ff"
        } else {
            fail("'fast_forward' must be defined and a boolean")
        }  
    } else {
        $arg_fast_forward = undef
    }
    
    if $fast_forward_only != undef {
        if is_bool($fast_forward_only) {
            $arg_fast_forward_only = "--ff-only"
        } else {
            fail("'fast_forward_only' must be defined and a boolean")
        }  
    } else {
        $arg_fast_forward_only = undef
    }
    
    if $all != undef {
        if is_bool($all) {
            $arg_all = "--all"
        } else {
            fail("'all' must be defined and a boolean")
        }  
    } else {
        $arg_all = undef
    }
    
    if $force != undef {
        if is_bool($force) {
            $arg_force = "--force"
        } else {
            fail("'force' must be defined and a boolean")
        }  
    } else {
        $arg_force = undef
    }
    
    if $verbose != undef {
        if is_bool($verbose) {
            $arg_verbose = "--quiet"
        } else {
            fail("'verbose' must be defined and a boolean")
        }  
    } else {
        $arg_verbose = undef
    }
    
    if $quiet != undef {
        if is_bool($quiet) {
            $arg_quiet = "--quiet"
        } else {
            fail("'quiet' must be defined and a boolean")
        }  
    } else {
        $arg_quiet = undef
    }
    
    if $http_proxy != undef and is_string($http_proxy) {
        $arg_http_proxy = "http_proxy=$http_proxy"
    } else {
        $arg_http_proxy = ""
    }
    
    if $https_proxy != undef and is_string($https_proxy) {
        $arg_https_proxy = "https_proxy=$https_proxy"
    } else {
        $arg_https_proxy = ""
    }
    
    require ::bi_git
    require ::bi_coreutils::stat::params
    
    $args = join(delete_undef_values([
        $arg_all,
        $arg_commit,
        $arg_fast_forward,
        $arg_fast_forward_only,
        $arg_force,
        $arg_quiet, 
        $arg_verbose,
    ]), " ")
    
    $command = "cd $local_dir && $arg_http_proxy $arg_https_proxy $::bi_git::exe_path pull $args $remote $refspec"
    
    bi_bash::exec { "$user/$command":
        command => $command,
        user => $user,
        group => $user,
    }
}